# --- HySoP Install Process ---
#
# By default, hysop libraries and python modules will be installed
# in the usual 'user' path of python (see https://docs.python.org/2/install/)
# as if you run "python setup.py install --user".
# This directory corresponds to site.USER_SITE variable.
#
# But if you are using virualenv, USER_SITE is not enable.
# In that case install will be done in site-packages of the virtualenv.
#
# You can also set your own install path using CMAKE_INSTALL_PREFIX option.
# In that case a proper set of PYTHONPATH environment variable will
# be required for hysop to work.
#
# Summary :
# cmake path-to-hysop-sources
# make install
# ---> install in USER_SITE (no virtualenv case)
# ---> install in site-packages of your virtualenv
#
# cmake -DCMAKE_INSTALL_PREFIX=/some/install/path
# make install
# ---> install in /some/install/path
# If /some/install/path is not a standard path of your system,
# you'll probably need something like :
# export PYTHONPATH=${PYTHONPATH}:/some/install/path
#
#

# Function to compute the install path/options
#
# Usage :
# set_install_options()
#
# Resutl :
# - set CMAKE_INSTALL_PREFIX (directory where lib and packages
# will be installed)
# - set install-opt : list of options given to 'python setup.py install' command.
#
function(set_install_options)
  # CMAKE_INSTALL_PREFIX given in cmake options :
  if(NOT restart)
    if(NOT CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT)
      set(restart 1 CACHE INTERNAL "True if this is not the first cmake run.")
      # if 'CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT' is false, it means
      # that -DCMAKE_INSTALL_PREFIX=something has been given to cmake.
      # A prefix is given by user. installation in PREFIX
      # (as python --prefix=<PREFIX>).
      set(install-opt "--prefix=${CMAKE_INSTALL_PREFIX}" CACHE INTERNAL
	"options for python installer")
      # Need to set the CMAKE_INSTALL_PREFIX to the proper site
      # (subdirectory of PREFIX)
      # Get python install suffix (~> lib/pythonX.X/site-packages) from numpy package
      set(PYTHON_COMMAND_GET_INSTALL_DIR
	"import os, re, numpy; print os.path.join(os.path.join(\"${CMAKE_INSTALL_PREFIX}\",*re.compile('/numpy/__init__.py.*').sub('',numpy.__file__).split('/')[-3:]), \"${PROJECT_NAME}\")")
    else()
      # Default case, check virtualenv
      set(restart 1 CACHE INTERNAL "True if this is not the first cmake run.")
      # First, we need to check if '--user' option works in the current environment.
      execute_process(COMMAND ${PYTHON_EXECUTABLE} -c
	"import site; print site.ENABLE_USER_SITE" OUTPUT_VARIABLE ENABLE_USER)
      string(STRIP ${ENABLE_USER} ENABLE_USER)
      if(ENABLE_USER)
	# Default prefix (as python --user) -> installation in site.USER_SITE
	execute_process(COMMAND ${PYTHON_EXECUTABLE} -c
	  "import site; print site.USER_BASE" OUTPUT_VARIABLE USER_BASE)
	set(install-opt "--prefix=${USER_BASE}" CACHE INTERNAL "")
	# Need to set the CMAKE_INSTALL_PREFIX to site.USER_SITE
	# Get python user site and install path = USER_SITE + project_name
	set(PYTHON_COMMAND_GET_INSTALL_DIR
	  "import site, os, sys ; print os.path.join(site.USER_BASE, os.path.join(\"lib\", os.path.join(\"python\" + str(sys.version_info.major) + '.' + str(sys.version_info.minor), os.path.join(\"site-packages\", \"${PROJECT_NAME}\"))))")
      else()
	# user site not included in the path,
	# which probably means that python is run using virtualenv
        # Command to find 'global' site-packages
	set(GET_SITE_PACKAGE
	  "from distutils.sysconfig import get_python_lib; print(get_python_lib())")
	execute_process(COMMAND ${PYTHON_EXECUTABLE} -c
	  "${GET_SITE_PACKAGE}" OUTPUT_VARIABLE GLOBAL_SITE_PACKAGE)
	string(STRIP ${GLOBAL_SITE_PACKAGE} GLOBAL_SITE_PACKAGE)
	set(PYTHON_COMMAND_GET_INSTALL_DIR ${GET_SITE_PACKAGE})
	set(install-opt "" CACHE INTERNAL "")
      endif()
    endif()
    # Set the CMAKE_INSTALL_DIR to the proper path
    execute_process(
      COMMAND ${PYTHON_EXECUTABLE} -c "${PYTHON_COMMAND_GET_INSTALL_DIR}"
      OUTPUT_VARIABLE ${PROJECT_NAME}_INSTALL_DIR)
    string(STRIP ${${PROJECT_NAME}_INSTALL_DIR} ${PROJECT_NAME}_INSTALL_DIR)

    set(CMAKE_INSTALL_PREFIX ${${PROJECT_NAME}_INSTALL_DIR}
      CACHE PATH "default install path" FORCE)

  endif()

endfunction()


### Start the configuration for installation ###

set_install_options()


# Target to remove hysop from install-path.
if(CMAKE_PATCH_VERSION LESS 12)
  set(${PROJECT_NAME}_PYTHONPATH ${CMAKE_INSTALL_PREFIX}/..)
  get_filename_component(${PROJECT_NAME}_PYTHONPATH ${${PROJECT_NAME}_PYTHONPATH} REALPATH)
else()
  set(${PROJECT_NAME}_PYTHONPATH ${CMAKE_INSTALL_PREFIX})
  get_filename_component(${PROJECT_NAME}_PYTHONPATH ${${PROJECT_NAME}_PYTHONPATH} DIRECTORY)
endif()
set(${PROJECT_NAME}_PYTHONPATH ${${PROJECT_NAME}_PYTHONPATH} CACHE PATH "PYTHONPATH for hysop")
add_custom_target(uninstall COMMAND rm -rf ${${PROJECT_NAME}_PYTHONPATH}/${PYPACKAGE_NAME}*
  COMMENT "Remove ${${PROJECT_NAME}_PYTHONPATH}/${PYPACKAGE_NAME} directory (hysop package and its dependencies)")
# To install python package AND hysop library and modules
add_custom_target(python-install COMMAND ${PYTHON_EXECUTABLE} ${CMAKE_CURRENT_BINARY_DIR}/setup.py install ${install-opt} config_fc --f90exec=${CMAKE_Fortran_COMPILER}
  #COMMAND make install
  WORKING_DIRECTORY ${CMAKE_CURRENT_BINARY_DIR} COMMENT "build/install hysop package")

install(CODE "execute_process(COMMAND ${CMAKE_BUILD_TOOL} python-install WORKING_DIRECTORY \"${CMAKE_CURRENT_BINARY_DIR}\")")

